package dev.fyloz.colorrecipesexplorer.dtos.account

import com.fasterxml.jackson.annotation.JsonIgnore
import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.SpringUserDetails
import dev.fyloz.colorrecipesexplorer.dtos.EntityDto
import dev.fyloz.colorrecipesexplorer.model.account.Permission
import dev.fyloz.colorrecipesexplorer.model.account.toAuthority
import java.time.LocalDateTime
import javax.validation.constraints.NotBlank
import javax.validation.constraints.Size

data class UserDto(
    override val id: Long = 0L,

    val firstName: String,

    val lastName: String,

    @field:JsonIgnore
    val password: String = "",

    val group: GroupDto?,

    val permissions: List<Permission>,

    val explicitPermissions: List<Permission> = listOf(),

    val lastLoginTime: LocalDateTime? = null,

    @field:JsonIgnore
    val isDefaultGroupUser: Boolean = false,

    @field:JsonIgnore
    val isSystemUser: Boolean = false
) : EntityDto {
    @get:JsonIgnore
    val fullName = "$firstName $lastName"
}

data class UserSaveDto(
    val id: Long = 0L,

    @field:NotBlank
    val firstName: String,

    @field:NotBlank
    val lastName: String,

    @field:NotBlank
    @field:Size(min = 8, message = Constants.ValidationMessages.PASSWORD_TOO_SMALL)
    val password: String,

    val groupId: Long?,

    val permissions: List<Permission>,

    @field:JsonIgnore
    val isSystemUser: Boolean = false,

    @field:JsonIgnore
    val isDefaultGroupUser: Boolean = false
)

data class UserUpdateDto(
    val id: Long = 0L,

    @field:NotBlank
    val firstName: String,

    @field:NotBlank
    val lastName: String,

    val groupId: Long?,

    val permissions: List<Permission>
)

data class UserLoginRequestDto(val id: Long, val password: String)

class UserDetails(
    val id: Any,
    private val username: String,
    private val password: String,
    val groupId: Long?,
    val permissions: Collection<Permission>
) : SpringUserDetails {
    constructor(user: UserDto) : this(user.id, user.fullName, user.password, user.group?.id, user.permissions)

    override fun getUsername() = username
    override fun getPassword() = password

    @JsonIgnore
    override fun getAuthorities() = permissions
        .map { it.toAuthority() }
        .toMutableList()

    override fun isAccountNonExpired() = true
    override fun isAccountNonLocked() = true
    override fun isCredentialsNonExpired() = true
    override fun isEnabled() = true
}
